﻿using System;
using System.Drawing;

using State.Events;
using State.Armor;
using State.Shooting;

using State.FactionInfo;

namespace State.Units.Step1
{
    public abstract class GameUnit
    {
        public enum UnitState
        {
            Normal,
            Dead
        }

        private UnitState _state;

        public event EventHandler<PositionChangedEventArgs> PositionChanged;

        public UnitSide Side { get; set; /* Can set, a unit can switch sides */ }
        public UnitBehaviour Behaviour { get; set; }

        public string Description { get; protected set; }
        public UnitType Type { get; protected set; }
        public IArmorBehaviour Armor { get; protected set; }
        public IShootingBehaviour ShootingBehaviour { get; protected set; }

        public Point Position { get; protected set; }

        protected FactionDetailsFactory FactionDetails { get; private set; }

        // Unit classes each take care of their own Display
        public abstract void Display();

        public GameUnit(FactionDetailsFactory factionFactory)
        {
            _state = UnitState.Normal;

            FactionDetails = factionFactory;

            Side = factionFactory.GetSide();
            Behaviour = factionFactory.GetBehaviour();
            Description = "Undefined Game Unit";
        }

        protected void OnPositionChanged()
        {
            if (PositionChanged != null)
            {
                PositionChanged(this, new PositionChangedEventArgs(Position));
            }
        }

        public void PerformShoot()
        {
            switch (_state)
            {
                case UnitState.Normal: 
                    ShootingBehaviour.Shoot();
                    break;
                case UnitState.Dead: 
                    Console.WriteLine("Can't shoot when dead!");
                    break;
            }
        }

        public void Move(Point newLocation)
        {
            switch (_state)
            {
                case UnitState.Normal: 
                    Position = newLocation;
                    OnPositionChanged();
                    break;
                case UnitState.Dead: 
                    Console.WriteLine("Can't move when dead!");
                    break;
            }
        }

        public void Kill()
        {
            // Kill the unit.
            Side = UnitSide.None;
            Description += ", Dead";

            _state = UnitState.Dead;
        }

        public override string ToString()
        {
            return string.Format("{0}, {1}", Type, _state);
        }
    }
}
